#!/usr/local/bin/perl -w

#### LAST UPDATED: Oct19, 2002 ####
# mistag.pl version 0.1

###########################################################################
#
# This program generates files showing all instances where the pre-tagged
# part of speech is overruled by the Brill Tagger.
# The program takes the pre-tagged file name(PRETAG), the brill tagged 
# file name(BRILL) and the names of the destination files
# (DESTINATION1 and DESTINATION2) as command line arguements.
# The output files have the line as in the pre-tagged file followed by the
# line as in the brill tagged file. Only those lines are shown where the
# tag of the head word has been updated by the Brill Tagger.
# The program creates two output files:
#	DESTINATION1 has all lines which meet the above requirement
#	DESTINATION2 has only those lines where the updated tag is in a 
# 		     different class from the pre-tag. For eg. an updation
#		     from a noun to verb etc but not from a common noun 
#		     to a proper noun etc. 
#
# USAGE : mistag.pl [OPTIONS] BRILL PRETAG DESTINATION1 DESTINATION2
#
###########################################################################

# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }

# Print help if asked
if ( $help )
{
    &help();
    exit;
}
# if version has been requested, show version
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}


# Check if command line options have been provided
# If not print usage information
if ( $#ARGV < 1 )
{
    &usage();
    exit;
}

# Open Files to be compared
$tgdfl = shift(@ARGV); 		
$ptgfl = shift(@ARGV);	
open(TGD,"<$tgdfl") or die "Could not open tagged file!\n";
open(PTG,"<$ptgfl") or die "Could not open pretag file!\n";
# Accept DESTINATION file names
$allfl = shift(@ARGV);
$radfl = shift(@ARGV);

if(-f $allfl)
{
    print "Destination1 file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y")
    {
        print "Overwriting the file.\n";
    }
    else
    {
        exit;
    }
}
if(-f $radfl)
{
    print "Destination2 file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y")
    {
        print "Overwriting the file.\n";
    }
    else
    {
        exit;
    }
}

# Open DESTINATION files
open(OP1,">$allfl") or die "Could not open file!\n";
open(OP2,">$radfl") or die "Could not open file!\n";

# Initialise line number
$lnnoptg=0; 
$allctr=0; $radctr=0; 
$write1=0; $write2=0;

# Read lines from pretagged file - line by line
while(<PTG>)
{
    $ln = $_;
    
    # place all multiple token data within square braces in curved braces
    # $ln =~ s/\[(\s?[\w\.]+\s[\w\.]+[^\]]*)\]/\($1\)/g;
    $ln =~ s/\[(\s*[^\]\s]+\s+[^\]\s]+\s*[^\]]*)\]/\($1\)/g;
    # Eliminate all square brace data
    $ln =~ s/\[[^\]]*\]//g;
    
    # Replace character references by appropriate symbols
    # --, -, ", &, <, >, ', [, ], {, }, #
    # These characters are tokenized as well
    while($ln =~ /&mdash;/g){
	$ln =~ s/&mdash;/ -- /g;}
    while($ln =~ /&ndash;/g){
	$ln =~ s/&ndash;/ - /g;}
    while($ln =~ /&([be]|[lr]d)?quo;/g){
	$ln =~ s/&([be]|[lr]d)?quo;/ " /g;}
    while($ln =~ /&amp;/g){
        $ln =~ s/&amp;/ & /g;}
    # while($ln =~ /&lt;/g){
    #    $ln =~ s/&lt;/ < /g;}
    # while($ln =~ /&gt;/g){
    #    $ln =~ s/&gt;/ > /g;}
    while($ln =~ /&apos;/g){
        $ln =~ s/&apos;/ ' /g;}
    while($ln =~ /&lsquo;/g){
        $ln =~ s/&lsquo;/ ' /g;}
    while($ln =~ /&rsquo;/g){
        $ln =~ s/&rsquo;/ ' /g;}
    while($ln =~ /&lsqb;/g){
        $ln =~ s/&lsqb;/ [ /g;}
    while($ln =~ /&rsqb;/g){
        $ln =~ s/&rsqb;/ ] /g;}
    while($ln =~ /&num;/g){
        $ln =~ s/&num;/ \# /g;}
    while($ln =~ /&lcub;/g){
        $ln =~ s/&lcub;/ \{ /g;}
    while($ln =~ /&rcub;/g){
        $ln =~ s/&rcub;/ \} /g;}

    # Eliminate all remaining character references
    while($ln =~ /&([^&;\.\s]*)[;\.]/g) {
        # Write character reference to filter file
        # print DB "$1\n";
        # Replace character references by space
        $ln =~ s/(&[^&;\.\s]*[;\.])/ /g; }

    # If line is blank read next line
    if($ln =~ /^\s*$/)
    { next; }
    # If line has only an xml tag, read next line
    if($ln =~ /^\s*<[^>]+>\s*$/)
    { next; }
    # If line has only &...; kind of tags, read next line
    # if($ln =~ /^\s*(&[^;]+;|\s*|\[[^\]]+\])+\s*$/)
    if($ln =~ /^\s*(&[^;\.\s]+[;\.]|\s*)+\s*$/)
    { next; }

    # Increment line number
    $lnnoptg++;
    # Check if line has head tag
    if($ln =~ /<head[^>]*>\s*([^<]+)\s*</)
    {
	# Capture the Pre-tag
	$pretag = $1;
	# Check if pretagging was done
	if($pretag =~ /\/\//)
	{ ;
	}
	else
	{
		# CHANGE May18: 
		# print "$ln\n\n";
		# print "Pretagging not done for atleast one line.\n";
		# exit();
		next;
	}
	# Read lines from Brill tagged file - line by line
	while($tgdln = <TGD>)
	{
	    # If line is blank read next line 
	    if(/^\s*$/)
	    { next; }
	    # CHANGE FEB17: to handle multiple head words
	    $tgdln =~ s/([^\/\s])\/[^\/\s]+/$1/g;
	
	    # Increment Brill tagged file line number 
	    $lnnotgd++;
	    # On reaching corresponding line number to 
	    # pre-tagged file line exit from loop
	    if($lnnoptg==$lnnotgd)
	    { last; }
	}

	# If tags match; read next line from pre-tagged file	
	# if($tgdln =~ /$pretag/)
	if($tgdln =~ /$pretag[\s\n]/)
	{ next; }
	# Increment counter of changes
	$allctr++;
	# Print line number and line of pretagged file
	# Print line of Brill tagged file - in DESTINATION1
	print OP1 "$lnnoptg  $_\n$tgdln\n\n\n";
	$write1=1;
	# Check for radical change	
	$pretag =~ s/([^\/]+\/\/[A-Z])[A-Z]+/$1/;
	if($tgdln =~ /$pretag/)
	{ next; }
	# Increment radical change counter
	$radctr++;
	# Print lines in DESTINATION2 file aswell
	print OP2 "$lnnoptg  $_\n$tgdln\n\n\n";
	$write2=1;
    }
}
if($write1 == 0)	{ unlink "$allfl"; }
if($write2 == 0)        { unlink "$radfl"; }

# Print number of changes and radical changes of tag
print "\n$allctr all-changes";
print "\n$radctr radical-changes\n\n";

# Subroutine to print help message
sub help
{
    print "Usage: mistag.pl [OPTIONS] BRILL PRETAG DESTINATION1 DESTINATION2 \n\n"; 
    print "This program generates files showing all instances where the pre-tagged part of speech is overruled by the Brill Tagger. The program takes the pre-tagged file (PRETAG), the brill tagged file (BRILL) and the destination files (DESTINATION1 and DESTINATION2) as command line arguements. The output files have the line as in the pre-tagged file followed by the line as in the brill tagged file. Only those lines are shown where the tag of the pre-tagged word has been updated by the Brill Tagger. The program creates two output files:\n\tDESTINATION1 has all lines which meet the above requirement\n\tDESTINATION2 has only those lines where the updated tag is in a different class from the pre-tag. For eg. an updation from a noun to verb etc but not from a common noun to a proper noun etc\n\n"; 
    print "OPTIONS:\n\n";
    print "  --help             Prints the help message.\n";
    print "  --version          Prints the version number.\n\n";
}

sub usage
{
    print "Usage:  mistag.pl [OPTIONS] BRILL PRETAG DESTINATION1 DESTINATION2 \n";
    AskHelp();
}
# Message describing way to get help
sub AskHelp
{
    print "Type mistag.pl --help for help.\n";
}
# function to output the version number
sub showVersion
{
    print STDERR "mistag.pl      -        version 0.1\n";
    print STDERR "A component of posSenseval version 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad Ted Pedersen\n";
}

